﻿using Infragistics.Controls.Charts;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;
using Infragistics.Olap.Xmla;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace XamDataChartOlapFlickerCase
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        FlatDataSource fds;
        SalesDataSample sds;
        string fileName = "customizations.txt";
        public MainWindow()
        {
            InitializeComponent();            
            sds = new SalesDataSample();
            fds = new FlatDataSource() { ItemsSource = sds, MeasureListLocation = MeasureListLocation.Rows } ;

            fds.ResultChanged += Fds_ResultChanged;

            fds.Columns = XmlaDataSource.GenerateInitialItems("[Date].[Date]");
            fds.Measures = XmlaDataSource.GenerateInitialItems("AmountOfSale, NumberOfUnits");

            if (File.Exists(fileName))
            {
                using(FileStream fs = new FileStream(fileName, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                {
                    fds.LoadCustomizations(fs);
                }

                xAxis.DataSource = fds;
            }   


        }

        bool fromRefreshGrid = false;
        private void Fds_ResultChanged(object sender, ResultChangedEventArgs e)
        {
            if (!fromRefreshGrid)
            {
                //ExpandAll(); //Always expands everything, so commented for time being.
                fds.SetMeasureAggregator((IMeasureViewModel)fds.Measures[0], new DoubleAverageAggregator());

                fds.RefreshGrid();
                fromRefreshGrid = true;
            }
            else
            {
                fromRefreshGrid = false;
            }
        }

        private void refreshGridBtn_Click(object sender, RoutedEventArgs e)
        {
            FlatDataSource fds = xAxis.DataSource as FlatDataSource;

            if (fds != null)
            {
                fds.RefreshGrid();
            }
        }         

        private void expandAllBtn_Click(object sender, RoutedEventArgs e)
        {
            ExpandAll();
        }

        private void ExpandAll()
        {
            FlatDataSource fds = xAxis.DataSource as FlatDataSource;

            var filterViewModelsColumns = fds.Columns.Where(fvm => fvm is IFilterViewModel).ToList();
            var filterViewModelsRows = fds.Rows.Where(fvm => fvm is IFilterViewModel).ToList();


            for (int i = 0; i < filterViewModelsColumns.Count; i++)
            {
                var filterViewModel = filterViewModelsColumns[i] as IFilterViewModel;
                ExpandToLevel(filterViewModel.FilterMembers, filterViewModel.Hierarchy.Levels.Count - 1);

                FilterViewModelBase filterViewModelBase = filterViewModel as FilterViewModelBase;
                filterViewModelBase.IsDirty = true;

            }

            for (int i = 0; i < filterViewModelsRows.Count; i++)
            {
                var filterViewModel = filterViewModelsRows[i] as IFilterViewModel;
                ExpandToLevel(filterViewModel.FilterMembers, filterViewModel.Hierarchy.Levels.Count - 1);

                FilterViewModelBase filterViewModelBase = filterViewModel as FilterViewModelBase;
                filterViewModelBase.IsDirty = true;
            }

            fds.RefreshGrid();
        }


        private void ExpandToLevel(IEnumerable<IFilterMember> filterMembers, int levelDepth)
        {
            foreach (IFilterMember filterMember in filterMembers)
            {
                if (filterMember.Member.LevelDepth < levelDepth)
                {
                    filterMember.IsExpanded = true;
                    if (filterMember.Member.LevelDepth < levelDepth - 1)
                    {
                        this.ExpandToLevel(filterMember.FilterMembers, levelDepth);
                    }
                }
            }
        }

        private void saveBtn_Click(object sender, RoutedEventArgs e)
        {
            if (File.Exists(fileName))
            {
                File.Delete(fileName);
            }

            using (FileStream fs = new FileStream(fileName, FileMode.CreateNew, FileAccess.Write))
            {
                fds.SaveCustomizations(fs);
            }
        }            

        private void loadBtn_Click(object sender, RoutedEventArgs e)
        {
            if (File.Exists(fileName))
            {
                using(FileStream fs = new FileStream(fileName, FileMode.OpenOrCreate, FileAccess.Read))
                {
                    fds.LoadCustomizations(fs);
                }
            }
        }

        private void assignDataSourceBtn_Click(object sender, RoutedEventArgs e)
        {
            xAxis.DataSource = fds;
        }
    }
}
